
import os
import sys
import click

CONTEXT_SETTINGS = dict(auto_envvar_prefix='AIRMAIL')

class Context(object):
    def __init__(self):
        self.verbose = False
        self.home = os.getcwd()

    def log(self, msg, *args):
        """Logs a message to stderr."""
        if args:
            msg %= args
        click.echo(msg, file=sys.stderr)

    def vlog(self, msg, *args):
        """Logs a message to stderr only if verbose is enabled."""
        if self.verbose:
            self.log(msg, *args)

    def preamble_log(self, preamble, value):
        click.echo(click.style(preamble, fg="blue", bold=True) + ": " + value)

    def warn_log(self, msg):
        click.echo(click.style("Warn", fg="yellow", bold=True) + ": " + msg)

    def verbose_log(self, msg):
        if self.verbose:
            click.echo(click.style("Verbose", fg="white", bold=True) + ": " + msg)

    def info_log(self, msg):
        click.echo(click.style("Info", fg="green", bold=True) + ": " + msg)

    def add_line_break(self):
        click.echo("\n")

    def err_log(self, msg, *args):
        if args:
            msg %= args
        click.echo(click.style("Error: ", fg="red", bold=True) + msg)


pass_context = click.make_pass_decorator(Context, ensure=True)
cmd_folder = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                          'commands'))


class AirMailCLI(click.MultiCommand):
    def list_commands(self, ctx):
        rv = []
        for filename in os.listdir(cmd_folder):
            if filename.endswith('.py') and \
               filename.startswith('cmd_'):
                rv.append(filename[4:-3])
        rv.sort()
        return rv

    def get_command(self, ctx, name):
        try:
            if sys.version_info[0] == 2:
                name = name.encode('ascii', 'replace')
            mod = __import__('airmail.commands.cmd_' + name,
                             None, None, ['cli'])
        except ImportError:
            return
        return mod.cli

@click.command(cls=AirMailCLI, context_settings=CONTEXT_SETTINGS)
@click.option("-p", "--profile", help="The AWS profile to use", default=lambda: os.environ.get("AWS_PROFILE", None))
@click.option("-e", "--env", help="The environment to target", default=lambda: os.environ.get("ENV", ""))
@click.option("-d", "--dry-run", help="Commands run with this flag will not result in actual changes to your AWS configuration", default=lambda: os.environ.get("DRY_RUN", False))
@click.option("-c", "--config-file", help="Specify the config file to use when running the commands", default=lambda: os.environ.get("CONFIG_FILE", "config.yml"))
@click.option("-v", "--verbose", help="Run commands with verbose logging", default=lambda: os.environ.get("VERBOSE", False))
@pass_context
def cli(ctx, profile, env, dry_run, config_file, verbose):
    """A CLI for getting code into the cloud"""
    ctx.env = env
    ctx.profile = profile
    ctx.dry_run = bool(dry_run)
    ctx.verbose = bool(verbose)
    ctx.config_file = config_file



import os
import click
import boto3
import json
from airmail.cli import pass_context
from airmail.services.ecs_service import ECSService

@click.group(short_help='Create/modify ECS services')
@pass_context
def cli(ctx):
    if ctx.dry_run:
        ctx.warn_log("Running in dry run mode! Commands will not execute against AWS!")

    if ctx.env is "":
        ctx.env = click.prompt('In which environment would you like to deploy?', type=str)

    ecs_service = ECSService(env=ctx.env, profile=ctx.profile, config=ctx.config_file)
    ctx.ecs_service = ecs_service
    ctx.add_line_break()
    ctx.preamble_log("Current Env", ecs_service.get_env())
    ctx.preamble_log("Project Name", ecs_service.get_name())
    ctx.preamble_log("Service Name", ecs_service.get_service())
    ctx.preamble_log("Project Cluster", ecs_service.get_top_level_prop("cluster"))
    ctx.preamble_log("Project Family", ecs_service.get_top_level_prop("family"))
    ctx.add_line_break()

@cli.command(name = 'deploy-service', short_help='Creates or updates a service based off the deploy configuration file')
@pass_context
def deploy_service(ctx):
    ctx.info_log('Running deploy-service...')
    ctx.info_log('This command will create an ECS service or update an existing service.')
    service_exists = ctx.ecs_service.service_exists() # Does the service exist?
    if service_exists:
        if not ctx.dry_run:
            ctx.ecs_service.update_service()
        else:
            ctx.warn_log("Service exists but will not be updated due to dry run mode")
    else:
        if not ctx.dry_run:
            ctx.ecs_service.create_service()
        else:
            ctx.warn_log("Service does not exist but will not be created due to dry run mode")

@cli.command(name = 'register-task', short_help='Creates a task definition based off the deploy configuration file')
@pass_context
def register_task(ctx):
    ctx.info_log('Running register-task...')
    ctx.info_log('This command will create a task definition for project based off the deploy configuration file')
    ctx.add_line_break()
    task_definition = ctx.ecs_service.create_task_definition()
    if not ctx.dry_run:
        ctx.ecs_service.register_task_definition(task_definition)
    else:
        ctx.warn_log("Task not registered due to dry run flag")
        ctx.verbose_log("Generated task definition JSON:")
        ctx.verbose_log(json.dumps(task_definition))


@cli.command(name = 'build-image', short_help='Builds a Docker image and pushes to ECR based on your deploy configuration')
@pass_context
def buildImage(ctx):
    ctx.info_log("Running build-image...")
    ctx.info_log("This command will build the image for ")
    ctx.add_line_break()

    if not ctx.dry_run:
        built_tag = ctx.ecs_service.build_container_image()
        ctx.info_log("Built project image: " + built_tag)
    else:
        ctx.warn_log("Image not built due to dry run flag")
